// Author: Marko Šantić
// Web: http://www.omnisdata.com/omnigrid
// Email: marko@omnisdata.com
// Company: Omnisdata Ltd.
// Licence: MIT licence
// Required: Mootools 1.2
// Version: OmniGrid 1.2.6
// ****************************************************************************

var BaseGrid = new Class({
	Implements: [Events,Options],
	
	getOptions: function() {
		return {
			alternateRows: true,
			showHeader:true,
			sortHeader:false,
			resizeColumns:true,
			editOnDblClick:false,
			url:null,
			// pagination
			pagination:false,
			page:1,
			perPageOptions: [10, 20, 50, 100, 200],
			perPage:10,
			// dataProvider
			dataProvider:null,
			toolbarHeight:25
		};
	},
	
	initialize: function(container, options, data) {
		this.setOptions(this.getOptions(), options);
		if (this.options.columnModel) {
			this.adjustColumnModel();
		} else {
			return;
		}
		
		// Setting this.container variable to element holder in html
		this.container = $(container);
		if (!this.container)
			return;
		
		this.calculateParameters();
		if (this.options.url) {
			this.loadData();
		} else if (data) {
			this.setData(data);
		}
		
		this.draw();
		this.reset();
	},
	
	adjustColumnModel: function() {
		var defCommonAttribs = {width:100, sort: 'ASC'};
		var defCheckboxAttribs = {labelT: 'true', labelF: 'false', valueT: 1, valueF: 0};
		var defNumberAttribs = {decimalPlaces: 2};
		
		var cm = null;
		for (var i = 0; i < this.options.columnModel.length; i++) {
			cm = this.options.columnModel[i];
			cm = $merge({}, defCommonAttribs, cm);
			if (cm.dataType == 'number') {
				cm = $merge({}, defNumberAttribs, cm);
			}
			if (cm.dataType == 'boolean' || cm.dataType == 'checkbox') {
				cm = $merge({}, defCheckboxAttribs, cm);
			}
			this.options.columnModel[i] = cm;
		}
	},
	
	calculateParameters: function() {
		this.columnCount = this.options.columnModel.length;
		this.cellPaddingLeft = 0; // needed in order to calculate real column width
		this.cellPaddingRight = 0; // needed in order to calculate real column width
		this.sumWidth = 0;
		this.visibleColumns = 0; // razlikuje se od columnCount jer podaci za neke kolone su ocitani ali se ne prikazuju, npr. bitno kod li width
		
		// array of summarize widths of all visible columns before current one + width of current column:
		// this.sumWidths[2] = column1.width + column2.width + column3.width
		this.sumWidths = new Array();
		for (var c = 0; c < this.columnCount; c++) {
			var columnModel = this.options.columnModel[c];
			if (columnModel.hidden) {
				var len = this.sumWidths.length;
				if (len > 0) {
					this.sumWidths.push(this.sumWidths[len-1]);
				} else {
					this.sumWidths.push(0);
				}
			} else {
				this.sumWidth += columnModel.width;
				this.visibleColumns++;
				this.sumWidths.push(this.sumWidth);
			}
		}
	},

	getBodyHeight: function() {
		// da ukupna visina cijelog grida bude this.options.height za body moramo oduzeti header
		
		// header
		var headerHeight = this.options.showHeader ? 24+2 : 0;  //+2 radi bordera
		
		// toolbar
		var toolbarHeight = this.toolbarDiv ? this.toolbarDiv.getStyle('height').toInt() : 0;

		// pagination toolbar height 25px + 1px bottom border
		var paginationToolbar = this.options.pagination ? 26 : 0;
			
		return this.options.height-headerHeight-toolbarHeight-paginationToolbar-2; //+2 radi bordera
	},
	
	// ************************************************************************
	// ************************* Toolbar **************************************
	// ************************************************************************
	createToolbar: function() {		
		var tDiv = new Element('div');
		tDiv.addClass('tDiv');
		tDiv.setStyle('width', this.options.width-2); 
		tDiv.setStyle('height', this.toolbarHeight);
		
		var bt = this.options.buttons;
		for (var i = 0; i < bt.length; i++) {
			var fBt = new Element('div');
			tDiv.appendChild(fBt);
			if (bt[i].separator)
			{
				fBt.addClass('btnseparator');
				continue;
			}
			
			fBt.addClass('fbutton');
			
			var cBt = new Element('div');
			cBt.addEvent('click', bt[i].onclick.bind(this, [bt[i].bclass, this])); 
			cBt.addEvent('mouseover', function(){this.addClass('fbOver');}); 
			cBt.addEvent('mouseout', function(){this.removeClass('fbOver');}); 
			
			fBt.appendChild(cBt);
			
			var spanBt = new Element('span');
			spanBt.addClass(bt[i].bclass);
			spanBt.setStyle('padding-left', 20 );
			spanBt.set('html', bt[i].name);
			cBt.appendChild(spanBt);
		}
		return tDiv;
	},
	
	createHeader: function() {
		var hDiv = new Element('div');
		hDiv.addClass('hDiv');
		hDiv.setStyle('width', this.options.width-2); // -2 because of borders of parent div.omnigrid
		this.container.appendChild(hDiv);
		
		var hDivBox = new Element('div');
		hDivBox.addClass('hDivBox');
		hDiv.appendChild(hDivBox);
		
		for (var c = 0; c < this.columnCount; c++) {
			var columnModel = this.options.columnModel[c];
			var div = new Element('div');
			div.store('column', c);
			div.store('dataType', columnModel.dataType);
			div.addClass('th');
			hDivBox.appendChild(div);
			if (this.cellPaddingLeft == 0) {
				this.cellPaddingLeft = div.getStyle('padding-left').toInt();
				if (isNaN(this.cellPaddingLeft)) {
					this.cellPaddingLeft = 0;
				}
			}
			if (this.cellPaddingRight == 0) {
				this.cellPaddingRight = div.getStyle('padding-right').toInt();
				if (isNaN(this.cellPaddingRight)) {
					this.cellPaddingRight = 0;
				}
			}
			// -2 because of borders
			var columnWidth = columnModel.width-this.cellPaddingLeft-this.cellPaddingRight-2;
			if (!columnModel.hidden && columnWidth < this.cellPaddingLeft+this.cellPaddingRight+2) {
				columnWidth = this.cellPaddingLeft+this.cellPaddingRight+2+1;
				var newColumnWidth = columnWidth+this.cellPaddingLeft+this.cellPaddingRight+2;
				alert('Width property of the column [' + columnModel.header + '] to small (' + columnModel.width + '). Increased to ' + newColumnWidth + '.');
				columnModel.width = newColumnWidth;
				this.calculateParameters();
			} else if (columnModel.hidden) {
				columnWidth = 1;
				columnModel.width = 1;
			}
			div.setStyle('width', columnWidth);
			
			if (columnModel.hidden) {
				div.setStyle('display', 'none');
			}
			
			var header = columnModel.header;
			
			if (header)
				div.innerHTML = header;
		}
		// must add padding-left times visible columns to total width
		hDivBox.setStyle('width', this.sumWidth);
		if (!this.options.showHeader)
			hDiv.setStyle('display', 'none');
			
		return hDiv;
	},
	
	createColumnSizeDrag: function() {
		var cDrag = new Element('div');
		cDrag.addClass('cDrag');
		var toolbarHeight = this.options.buttons ? this.toolbarDiv.getStyle('height').toInt() : 0; // toolbar
		cDrag.setStyle('top', toolbarHeight);
		
		var dragTempWidth = 0;
		for (var c = 0; c < this.columnCount; c++) {
			var columnModel = this.options.columnModel[c];
			
			//if (columnModel.hidden) continue;
				
			var dragSt = new Element('div');
			
			//alert(dragTempWidth+' '+columnModel.width);
			// -(Browser.Engine.trident ? 10 : 0 )
			var headerHeight = this.options.showHeader ? 24+2 : 0; // +2 border
			
			dragSt.setStyles({top:1,left: dragTempWidth+columnModel.width, height: headerHeight, display:'block'}); // bodyHeight+
			dragSt.store('column', c);
			cDrag.appendChild(dragSt);
			
			// Events
			dragSt.addEvent('mouseout', this.outDragColumn.bind(this));
			dragSt.addEvent('mouseover', this.overDragColumn.bind(this));
			
			var dragMove = new Drag(dragSt, {snap:0}); // , {container: this.container.getElement('.cDrag') }
			dragMove.addEvent('drag', this.onColumnDragging.bind(this) );
			dragMove.addEvent('start', this.onColumnDragStart.bind(this) );
			dragMove.addEvent('complete', this.onColumnDragComplete.bind(this) );
			
			
			if (columnModel.hidden) 
				dragSt.setStyle('display', 'none');
			else
				dragTempWidth += columnModel.width;
		}

		return cDrag;
	},

	createPaginationDiv: function() {
		var pDiv = new Element('div');
		pDiv.addClass('pDiv');
		pDiv.setStyle('width', this.options.width-2); 
		pDiv.setStyle('height', 25);
		
		var pDiv2 = new Element('div');
		pDiv2.addClass('pDiv2');
		pDiv.appendChild(pDiv2);
		
		var h = '<div class="pGroup"><select class="rp" name="rp">';
		
		// *****
		var optIdx;
		var setDefaultPerPage = false;
		for (optIdx=0; optIdx<this.options.perPageOptions.length; optIdx++)
		{
			if (this.options.perPageOptions[optIdx] != this.options.perPage)
				h += '<option value="' + this.options.perPageOptions[optIdx] + '">' + this.options.perPageOptions[optIdx] +'</option>';
			else{
				setDefaultPerPage = true;
				h += '<option selected="selected" value="' + this.options.perPageOptions[optIdx] + '">' + this.options.perPageOptions[optIdx] +'</option>' ;
			}
		}
		// *****

		h += '</select></div>';
		
		h += '<div class="btnseparator"></div><div class="pGroup"><div class="pFirst pButton"></div><div class="pPrev pButton"></div></div>';
		h += '<div class="btnseparator"></div><div class="pGroup"><span class="pcontrol"><input class="cpage" type="text" value="1" size="4" style="text-align:center"/> / <span></span></span></div>';
		h += '<div class="btnseparator"></div><div class="pGroup"><div class="pNext pButton"></div><div class="pLast pButton"></div></div>';
		h += '<div class="btnseparator"></div><div class="pGroup"><div class="pReload pButton"></div></div>';
		h += '<div class="btnseparator"></div><div class="pGroup"><span class="pPageStat"></div>';
		
		if (this.options.filterInput) h += '<div class="btnseparator"></div><div class="pGroup"><span class="pcontrol"><input class="cfilter" type="text" value="" style="" /><span></div>';
		
		pDiv2.innerHTML = h;

		// set this.options.perPage value from this.options.perPageOptions array
		var rpObj = pDiv2.getElement('.rp');
		if (!setDefaultPerPage && rpObj.options.length>0)
		{
			this.options.perPage = rpObj.options[0].value;
			rpObj.options[0].selected = true;
		}
		// ********

		pDiv2.getElement('.pFirst').addEvent('click', this.firstPage.bind(this) );
		pDiv2.getElement('.pPrev').addEvent('click', this.prevPage.bind(this) );
		pDiv2.getElement('.pNext').addEvent('click', this.nextPage.bind(this) );
		pDiv2.getElement('.pLast').addEvent('click', this.lastPage.bind(this) );
		pDiv2.getElement('.pReload').addEvent('click', this.refresh.bind(this) );
		pDiv2.getElement('.rp').addEvent('change', this.perPageChange.bind(this));
		pDiv2.getElement('input.cpage').addEvent('keyup', this.pageChange.bind(this) );
		
		if (this.options.filterInput) pDiv2.getElement('input.cfilter').addEvent('change', this.firstPage.bind(this) ); // goto 1 & refresh
		
		return pDiv;
	},
	
	createBody: function() {
		var bDiv = new Element('div');
		bDiv.addClass('bDiv');
		
		if (this.options.width)
			bDiv.setStyle('width', this.options.width-2); // -2 because of borders

		var bodyHeight = this.getBodyHeight();
		bDiv.setStyle('height', bodyHeight);	

		//  scroll event
		this.onBodyScrollBind = this.onBodyScroll.bind(this);
		bDiv.addEvent('scroll', this.onBodyScrollBind);
		
		//alert(this.visibleColumns);
		this.ulBody = new Element('ul');
		this.ulBody.setStyle('width', this.sumWidth); // da se ne vidi visak, ul je overflow hidden
		bDiv.appendChild(this.ulBody);
		
		return bDiv;
	},
	
	// ************************************************************************
	// ************************* Main draw function ***************************
	// ************************************************************************
	draw: function() {
		this.removeAll(); // reset variables and only empty ulBody 
		this.container.empty(); // empty all 
		
		// ************************************************************************
		// ************************* Container ************************************
		// ************************************************************************
		if (this.options.width) this.container.setStyle('width', this.options.width);
		if (this.options.height) this.container.setStyle('height', this.options.height);
		this.container.addClass('omnigrid');
		
		// ************************************************************************
		// ************************** Toolbar *************************************
		// ************************************************************************
		if (this.options.buttons) {
			if (!this.toolbarDiv) {
				this.toolbarDiv = this.createToolbar();
			}
			this.container.appendChild(this.toolbarDiv);
		}
		
		// ************************************************************************
		// ************************* Header ***************************************
		// ************************************************************************
		if (this.options.showHeader) {
			if (!this.headerDiv) {
				this.headerDiv = this.createHeader();
			}
			this.container.appendChild(this.headerDiv);
		}
		
		// ************************************************************************
		// ************************* Column size drag *****************************
		// ************************************************************************
		if (this.options.resizeColumns)
		{
			if (!this.columnSizeDragDiv) {
				this.columnSizeDragDiv = this.createColumnSizeDrag();
			}
			this.container.appendChild(this.columnSizeDragDiv);
		}
		
		// ************************************************************************
		// ************************* Body *****************************************
		// ************************************************************************
		if (!this.bodyDiv) {
			this.bodyDiv = this.createBody();
		}
		this.container.appendChild(this.bodyDiv);
		
		// ************************************************************************
		// ************************* Pagination ***********************************
		// ************************************************************************
		if (this.options.pagination && !this.paginationDiv )
		{
			this.paginationDiv = this.createPaginationDiv();
			this.container.appendChild(this.paginationDiv);			
		}
	},

	renderData: function(){
		this.ulBody.empty();
		this.inlineEditSafe = null;
		
		// variables for scroll offsets on vertical scrollbar
		this.sumHeight = 0;
		this.sumHeights = new Array();
		var sumScrolledHeight = 0;
		var rowHeight = 24;
		var horizontalScrollbarHeight = 0;
		
		if (this.sumWidths && this.sumWidths[this.sumWidths.length-1] > this.options.width) {
			horizontalScrollbarHeight = 15;
		}
		
		if (this.options.data && this.options.data.length)
		{
			var columnCount = this.options.columnModel.length;
			var rowCount = this.options.data.length;
			
			for (var r=0; r<rowCount; r++)
			{
				var rowdata = this.options.data[r];
				
				var li = new Element('li');
				li.setStyle('width', this.sumWidth); // inace se Div-ovi wrapaju, a u IE nastaje cudan 1px border ispod LI el.
				li.store('row', r);
				li.addClass('tr');
				if (this.options.alternateRows) {
					if (r%2==1) {
						li.addClass('erow');
					}
				}
				li.addEvent('click', this.onRowClick.bind(this));
				li.addEvent('dblclick', this.onRowDblClick.bind(this));		
				li.addEvent('mouseover', this.onRowMouseOver.bind(this) );
				li.addEvent('mouseout',  this.onRowMouseOut.bind(this) );
			
				// Here we set scrolling offset for the current row
				this.sumHeight += rowHeight;
				this.sumHeights.push(this.sumHeight);
				
				this.ulBody.appendChild(li);
				
				if (this.options.tooltip)
				{
					this.options.tooltip.attach( tr );											
				}
				
				var firstvisible = -1;
				for (var c=0; c<columnCount; c++)
				{
					var columnModel = this.options.columnModel[c];
					var cellValue = rowdata[columnModel.dataIndex];
					
					//if (columnModel.hidden)
					//	continue;
					
					var div = new Element('div');
					div.addClass('td');
					div.store('column', c);
					div.store('row', r);
					div.addEvent('click', this.onCellClick.bind(this, div));
					var columnWidth = columnModel.width-this.cellPaddingLeft-this.cellPaddingRight-2;
					if (columnWidth < 0) {
						// this is hidden column
						columnWidth = 1;
					}
					div.setStyle('width', columnWidth); // -2 because of borders
					//div.setStyle('overflow-x', 'hidden');
						
					li.appendChild(div);

					firstvisible = (!columnModel.hidden && firstvisible == -1) ? c : firstvisible;
					
					var toggleicon = "";
					if (firstvisible==c && this.options.accordion && this.options.showtoggleicon)
					{					
						toggleicon = "<div class='toggleicon'></div>";
					}
					
					if (columnModel.hidden) div.setStyle('display', 'none');					
					
					if (columnModel.onMouseOver)
					{
						div.onmouseover = this.onMouseOver.bind(this, {element:div, columnModel:columnModel, data:rowdata });												
					}
					
					// title
					if (columnModel.title) div.title = rowdata[columnModel.title];

					
					if (columnModel.dataType == "checkbox" || columnModel.dataType == "boolean")
					{
						var cell_renderer = new Element('div');
						var input_id = 'input_'+c+'x'+r;
						var input = new Element('input', {type:"checkbox", id:input_id});
						if (cellValue == columnModel.valueT) {
							input.set('checked', 'on');
						}
						if (!columnModel.editable) {
							input.set('disabled', true);
						}
						
						var label = null;
						cell_renderer.appendChild(input);
						cell_renderer.addClass('table-cell');
						
						div.appendChild(cell_renderer);
						
						if (columnModel.labelT && columnModel.labelF) {
							cell_renderer = new Element('div');
							cell_renderer.addClass('table-cell');
							div.appendChild(cell_renderer);
							
							label = new Element('label', {'for':input_id})
							if (cellValue == columnModel.valueT) {
								label.innerHTML = columnModel.labelT;
							} else {
								label.innerHTML = columnModel.labelF;
							}
							cell_renderer.appendChild(label);
						}
						input.onchange = this.updateDataForControl.bind(this, {row:r, col:c, input:input, label:label});

					} else if (columnModel.dataType == "select")
					{
						var cell_renderer = new Element('div');
						var input_id = 'input_'+c+'x'+r;
						var input = new Element('select', {id:input_id});
						input.setStyle('width', columnModel.width - 12);
						for (var i = 0; i < columnModel.options.length; i++) {
							input.options.add(new Option(columnModel.options[i], columnModel.options[i], columnModel.options[i]==cellValue, columnModel.options[i]==cellValue));
						}
						if (!columnModel.editable) {
							input.set('disabled', true);
						}
						cell_renderer.appendChild(input);
						cell_renderer.addClass('table-cell');
						div.appendChild(cell_renderer);
						input.onchange = this.updateDataForControl.bind(this, {row:r, col:c, input:input});
					} else if (columnModel.type == "image") {
					/*	var img = new Element('img');
						img.src = this.options.data[r][columnModel.dataIndex];
						td.appendChild(img);*/
						
					}else if (columnModel.type == 'custom') {
						//columnModel.labelFunction(td, this.options.data[r], r);
					}else if (columnModel.labelFunction != null) {
							div.innerHTML = columnModel.labelFunction(rowdata, r, columnModel);
					}else {
							var str = new String(cellValue); // mora biti string, jer ako dode 0 kao broj error
							
							if (str == null || str == 'null' || str == 'undefined' || str == "" ) str = '&nbsp;';

							var trimmed = str.replace(/^\s+|\s+$/g, ''); // ako je prazan string
							if(trimmed.length==0) str = '&nbsp;';
							
							// Column text align propert.
							// moram prije srediti racunanje width radi padding:0 kad se aling
							//if (columnModel.align) div.setStyles({'text-align': columnModel.align, 'padding-left':0});
							
							div.innerHTML = toggleicon+str;
							
							// *** reg. event to toggleicon ***
							if (firstvisible==c && this.options.accordion && this.options.showtoggleicon)
							{
								div.getElement('.toggleicon').addEvent('click', this.toggleIconClick.bind(this));
							}
					}
					
				} // for (entire row created)
				// ***********************
				this.renderDataRowCreated(li, r);
			}
		}
	},
	
	updateDataForControl: function(params) {
		var cm = this.options.columnModel[params.col];
		if (params.input.type == 'checkbox') {
			if (params.input.get('checked')) {
				this.options.data[params.row][cm.dataIndex] = cm.valueT;
			} else {
				this.options.data[params.row][cm.dataIndex] = cm.valueF;
			}
			
			if (params.label) {
				if (this.options.data[params.row][cm.dataIndex]) {
					params.label.set('text', cm.labelT);
				} else {
					params.label.set('text', cm.labelF);
				}
			}
			return;
		}
		if (params.input.type == 'select-one') {
			this.options.data[params.row][cm.dataIndex] = params.input.value;
		}
	},
	
	renderDataRowCreated: function(li, r) {},
	
	reset: function(){
		this.renderData();
		
		this.refreshDelayID = null;
		this.dragging = false;
		this.selected = new Array();
		
		if (!this.boundKeyDown && this.onKeyDown) {
			this.boundKeyDown = this.onKeyDown.bind(this);
			window.addEvent('keydown', this.boundKeyDown);
		}
		
		
		// ******************************************************************
		// **************************** Setup header ************************
		// ******************************************************************
		this.container.getElements('.th').each(function(el,i){
			//alert(el.dataType);
			var dataType = el.retrieve('dataType');
			if(dataType){
				//
				el.getdate = function(str){
					// inner util function to convert 2-digit years to 4
					function fixYear(yr) {
						yr = parseInt(yr);
						if (yr<50) { yr += 2000; }
						else if (yr<100) { yr += 1900; }
						return yr;
					};
					var ret;
					if (str.length>12){
						strtime = str.substring(str.lastIndexOf(' ')+1);
						strtime = strtime.substring(0,2)+strtime.substr(-2)
					}else{
						strtime = '0000';
					}
					//
					// YYYY-MM-DD
					if (ret=str.match(/(\d{2,4})-(\d{1,2})-(\d{1,2})/)) {
						return (fixYear(ret[1])*10000) + (ret[2]*100) + ret[3] + strtime;
					}
					// DD/MM/YY[YY] or DD-MM-YY[YY]
					if (ret=str.match(/(\d{1,2})[\/-](\d{1,2})[\/-](\d{2,4})/)) {
						return (fixYear(ret[3])*10000) + (ret[2]*100) + ret[1] + strtime;
					}
					return 999999990000; // So non-parsed dates will be last, not first
				};
				
				//
				el.findData = function(elem){
					var child = elem.getFirst();
					if(child){
						return el.findData(child);
					}else{
						return elem.innerHTML.trim();
					}
				};
				
				//
				el.compare = function(a, b){
					// a i b su LI elementi

					var var1 = a.getChildren()[i].innerHTML.trim();
					var var2 = b.getChildren()[i].innerHTML.trim();
					
					//console.log(el.sortBy);
					
					//var1 = a.getChildren()[i].firstChild.data;
					//var2 = b.getChildren()[i].firstChild.data;
					
					if(dataType == 'number' || dataType == 'integer'){
						var1 = parseFloat(var1);
						var2 = parseFloat(var2);
						
						if(el.sortBy == 'ASC'){
							return var1-var2;
						}else{
							return var2-var1;
						}
						
					}else if(dataType == 'string'){
						var1 = var1.toUpperCase();
						var2 = var2.toUpperCase();
						
						if(var1==var2){return 0};
						if(el.sortBy == 'ASC'){
							if(var1<var2){return -1};
						}else{
							if(var1>var2){return -1};
						}
						return 1;
						
					}else if(dataType == 'date'){
						var1 = parseFloat(el.getdate(var1));
						var2 = parseFloat(el.getdate(var2));
						
						if(el.sortBy == 'ASC'){
							return var1-var2;
						}else{
							return var2-var1;
						}
						
					}else if(dataType == 'currency'){
						var1 = parseFloat(var1.substr(1).replace(',',''));
						var2 = parseFloat(var2.substr(1).replace(',',''));
						
						if(el.sortBy == 'ASC'){
							return var1-var2;
						}else{
							return var2-var1;
						}
						
					}
					
				}

			}
		}, this);
	},

	getEditorParams: function(options) {
		var editorParams = {c:null, row:null, li:null, colmod:null, td:null, data:null};
		var sels = this.getSelectedIndices();

		if ( !sels || sels.length==0 || !this.options.editable ) return editorParams;
		
		editorParams.row = sels[0];
		editorParams.li = this.elements[ sels[0] ];
		
		// nadi index u columnModel
		editorParams.c = options.columnIndex ? options.columnIndex : 0; // ako ne nade ili nije definiran pretpostavi 1.
		if (options.dataIndex)
		{
			for (; editorParams.c < this.options.columnModel.length; editorParams.c++) 
			{
				editorParams.colmod = this.options.columnModel[editorParams.c];
				
				if (editorParams.colmod.hidden) continue;
				//console.log(colmod.dataIndex+" "+options.dataIndex);
				if ( editorParams.colmod.dataIndex == options.dataIndex ) break;
			}
		}
		
		if (editorParams.c == this.options.columnModel.length) return editorParams; // znaci da nije nasao columnu

		editorParams.colmod = this.options.columnModel[editorParams.c];

		if (!editorParams.colmod.editable) return editorParams;
		
		editorParams.td = editorParams.li.getElements('div.td')[editorParamsc];
		editorParams.data = this.options.data[ sels[0] ];
		return editorParams;
	},
	
	// API	
	// pretvara zadanu columnu u inline edit mode
	// options = {
	//		dataIndex:Number - column name || columnIndex:Number - column index 
	//}
	edit: function(options) {
		if (this.inlineeditmode) return;
		
		// if options not defined then we edit activeCell
		var editorParams = this.getEditorParams(options);
		if (editorParams.c == null || editorParams.row == null || !editorParams.li || !editorParams.colmod || !editorParams.td || !editorParams.data) {
			return;
		}
		var li = editorParams.li;
		var colmod = editorParams.colmod;
		var td = editorParams.td;
		var data = editorParams.data;
		var c = editorParams.c;
		var row = editorParams.row;

		if (colmod.dataType == 'checkbox' || colmod.dataType == 'boolean' || colmod.dataType == 'select') return;
		var width = td.getStyle('width').toInt()-this.cellPaddingLeft-this.cellPaddingRight-2-4;
		var height = 15;
		var html = data[colmod.dataIndex];

		td.innerHTML = "";
		var input = new Element('input', {style:"width: "+width+"px; height: "+height+"px;", maxlength:254, value: html} );

		input.addClass('inline');
		input.addEvent("keydown", this.finishEditing.bind(this) );
		input.addEvent("blur", this.finishEditing.bind(this) );
		input.inject(td);
		input.focus();
		input.select();
		
		//td.innerHTML = " onkeypress=\"return fieldEnter(this,event,' actual.id ')\" onfocus=\"highLight(this);\" onblur=\"noLight(this); return fieldBlur(this,' actual.id ');\" />";
		//td.innerHTML = '';
		
		this.inlineEditSafe = {row:row, columnModel: colmod, td:td, input:input, oldvalue: html};

		this.inlineeditmode = true; // radi chromea jer poziva keyup i blur event skoro paralelno !?
		
		return this.inlineEditSafe;
		
		// global click za finish editing --> ne treba jer ima onblur event
		//(Browser.Engine.trident ? document : window).addEvent("click", this.finishEditing.bind(this) );
	},
	
	finishEditing: function(evt)
	{
		if (!this.inlineeditmode) return;
		
		if ( evt && evt.type == "keydown" && evt.key != 'enter' && evt.key != 'esc' ) return;
		evt.stop();
		
		//if (evt) console.log(evt.type+" "+this.inlineEditSafe.oldvalue+" "+this.editmode);
		
		this.inlineeditmode = false;  // radi chromea

		var row = this.inlineEditSafe.row;
		var data = this.options.data[ row ];
		var colmod = this.inlineEditSafe.columnModel;
		var td = this.inlineEditSafe.td;
		

		// if enter update cell value, if not restore previous one
		data[colmod.dataIndex] = ( evt && evt.type == "keydown" && evt.key == 'enter') ? this.inlineEditSafe.input.value : this.inlineEditSafe.oldvalue;
		
		// update html content
		td.innerHTML = colmod.labelFunction ? colmod.labelFunction(data, row, colmod) : data[colmod.dataIndex];
		if (td.innerHTML.length == 0) td.innerHTML = "&nbsp;"; // bitno jer inace ne bi drugi put reagirao na dbl click
		
		// if enter and value changed, fire "editcomplete" event
		if ( evt && evt.type == "keydown" && evt.key == 'enter' && this.inlineEditSafe.oldvalue != td.innerHTML) {
			this.inlineEditSafe.target = this; // ovo je mozda lose jer poslije brisem varijablu this.inlineEditSafe
			this.fireEvent("editcomplete", this.inlineEditSafe);
		}
		
		this.inlineEditSafe = null;		
	},
	
	toggleDisplay: function(el){
		if (el.getStyle('display') == 'block')
		{
			el.setStyle('display', 'none');
		} else {
			el.setStyle('display', 'block');
		}
	},

	getLiParent: function (target){
		// ! ako se koristi labelFunction onda neki html elem. moze hvatati event, detektiraj pravi li
		target = $(target);

		while ( target && !target.hasClass('td') ){
			target = target.getParent();
		}
		
		if (target)
			return target.getParent();
	},
	
	onRowMouseOver: function (evt){
		var li = this.getLiParent(evt.target);
		if (!li) return;
		
		if (!this.dragging)
			li.addClass('over');
			
		this.fireEvent("mouseover", {target:this, row:li.retrieve('row'), element:li });
	},
	
	onRowMouseOut: function (evt){
		var li = this.getLiParent(evt.target);
		if (!li) return;
		
		if (!this.dragging)
			li.removeClass('over');
			
		this.fireEvent("mouseout", {target:this, row:li.retrieve('row'), element:li });
	},
	
	onCellClick: function(div, evt) {
		if (this.inlineeditmode) this.finishEditing();
	},
	
	onRowClick: function (evt){
	
		var li = this.getLiParent(evt.target);
		
		if (!li) return;
		
		
		if (this.options.selectable)
		{
			var currentindex = li.retrieve('row');
			var selectedNum = this.selected.length;
			var dontselect = false;
			
			if ( (!evt.control && !evt.shift) || !this.options.multipleSelection )
			{
				// ocisti stari selection
				this.elements.each(function(el, i){ el.removeClass('selected') }, this);
				
				//for (var i=0; i<this.selected.length; i++) this.elements[ this.selected[i] ].removeClass('selected');
				
				this.selected = new Array();
			}
		
			if ( evt.control )
			{
				for (var i=0; i<selectedNum; i++)
				{
					if ( currentindex == this.selected[i] ) // ako je vec selektiran unselectaj ga
					{
						this.elements[ currentindex ].removeClass('selected');
						this.selected.splice(i, 1 );
						dontselect = true;
					}
				}
			}
		
			if ( evt.shift && this.options.multipleSelection )
			{
				var si = 0;
				if (this.selected.length>0)
					si = this.selected[selectedNum-1]; // uzmi zadnjeg
					
				var endindex = currentindex;
				
				startindex = Math.min(si, endindex);
				endindex = Math.max(si, endindex);
				
				//if ( !evt.control ) this.unselectAll(); // ako ne drzi shift+ctrl ocisti select
				for (var i=startindex; i<=endindex; i++)
				{
					this.elements[i].addClass('selected');
					this.selected.push( Number(i) );
				}
			}
					
			if (!dontselect)
			{
				li.addClass('selected');
				this.selected.push( Number(li.retrieve('row')) );
			}
			
			this.unique(this.selected, true); // izbaci sve duplikate iz selection arr.
		}
		
		//console.log(this.selected);
	},

	onKeyDown: null,
	
	getPrevVisibleColumn: function(column) {
		for (var i = column - 1; i < this.options.columnModel.length && i >= 0; i--) {
			var cm = this.options.columnModel[i];
			if (!cm.hidden) {
				return i;
			}
		}
		return null;
	},

	getNextVisibleColumn: function(column) {
		for (var i = column + 1; i < this.options.columnModel.length; i++) {
			var cm = this.options.columnModel[i];
			if (!cm.hidden) {
				return i;
			}
		}
		return null;
	},

	// returns width of all columns counting from 0 to columnNumber inclusive
	getWidthOfColumns: function(columnNumber)
	{
		if (columnNumber < this.scrollData.sumWidths.length) {
			return this.sumWidths[columnNumber];
		}
		return null;
	},

	onRowDblClick: function (evt){
		
		var li = this.getLiParent(evt.target);
		if (!li) return;
		
		var t = evt.target;
		if ( this.options.editable && this.options.editondblclick && t.hasClass('td') )
		{
			var childs = li.getChildren();
			for(var i=0; i<childs.length; i++)
			{
				if (childs[i] == t) break;
			}
			
			var obj = this.edit({columnIndex:i});
			
			if (obj) obj.input.selectRange(0, obj.input.value.length);
		}
	
		if (this.options.accordion && this.options.openAccordionOnDblClick)
		{
			this.accordianOpen(li);
		}
		
		this.fireEvent("dblclick", {row:li.retrieve('row'), target:this, element:li});
	},
	
	onLoadData: function (data)
	{
		if (data) {
			//this.hideLoader(); //u setData() jer mu nekad dosta treba da nacrta
			this.setData(data);
			// API
			this.fireEvent("loaddata", {target:this, data:data, pkey:data.pkey});	// jedino pkey salje van jer se on nigdje ne sprema trenutno unutar OMG
		}
	},
		
	unique: function(a, asNumber)
	{
		function om_sort_number(a, b)
		{
			return a - b;
		}
		
		var sf =  asNumber ? om_sort_number : function(){};
		
		a.sort( sf );
		
		for (var i = 1; i < a.length; i++)
		{
			if (a[i-1] == a[i])
			{
				a.splice(i, 1);
				i--;
			}
		}
		
		return a;
	},
	
	// API
	loadData: function (url)
	{
		if (!url && !this.options.url && !this.options.dataProvider)
			return;

		var param = {};
		
		// ************* pagination *************************
		if (this.options.pagination)
			param = {page:this.options.page, perpage:this.options.perPage};
		
		// ************* server sorting *********************
		if (this.options.serverSort){
			param.sorton = this.options.sortOn;
			param.sortby = this.options.sortBy;
		}
		
		
		if (this.options.filterInput) 
		{
			var cfilter = this.container.getElement('input.cfilter');
			if (cfilter) param.filter = cfilter.value;
		}
		
		this.showLoader();
		
		if (this.options.dataProvider)
		{
			// load data throw external class
			this.options.dataProvider.loadData(param);
		}else
		{
			var url = (url != null) ? url : this.options.url;
			var request = new Request.JSON({method:'GET', url:url, data:param});
			request.addEvent("complete", this.onLoadData.bind(this));
			request.get();
		}
	},
	
	// API
	refresh: function(){
		this.loadData();
	},
		
	// API
	setData: function(data, cm)
	{
		if (!data) return;
		this.options.data = data.data;
		// ako nije zadana columnModel napravi ga sam
		if (!this.options.columnModel) this.setAutoColumnModel();

		if (this.options.pagination)
		{
			this.options.page =  data.page*1;
			this.options.total =  data.total;
			this.options.maxpage = Math.ceil(this.options.total/this.options.perPage);
			
			this.container.getElement('div.pDiv input').value = data.page;
			var to = (data.page*this.options.perPage) > data.total ? data.total : (data.page*this.options.perPage);
			this.container.getElement('div.pDiv .pPageStat').set('html', ((data.page-1)*this.options.perPage+1)+'..'+to+' / '+data.total);
			this.container.getElement('div.pDiv .pcontrol span').set('html', this.options.maxpage);
		}
			

		if (cm)
		{
			// first check is new columnModel different from active one
			if (this.options.columnModel != cm) 
			{
				this.setColumnModel(cm);
			}
		}
		if (this.bodyDiv) {
			this.reset();
		}
		this.hideLoader(); // kad je zavrsio sa loadanjem i crtanjem tek onda makni loader
	},
	
	// API
	getData: function(){
		return this.options.data;
	},
	
	// API
	getDataByRow: function(row){
		if (row >=0 && row<this.options.data.length) {
			var datarow = this.options.data[row];
			if (datarow) {
				for (var i = 0; i < this.options.columnModel.length; i++) {
					var cm = this.options.columnModel[i];
					if (cm.dataType == 'number') {
						datarow[cm.dataIndex] = parseFloat(datarow[cm.dataIndex]);
					} else if (cm.dataType == 'number') {
						datarow[cm.dataIndex] = parseInt(datarow[cm.dataIndex]);
					}
				}
			}
			return datarow;
		}
		return null;
	},
	
	// API
	setDataByRow: function(row, data){
		if (row >=0 && row<this.options.data.length) {
			this.options.data[row] = data;
			var li = this.elements[row];
			var cells = li.getElements('div.td');
			for (var i = 0; i < this.options.columnModel.length; i++) {
				var cm = this.options.columnModel[i];
				if (cm.dataType == 'number' && cm.decimalPlaces > 0) {
					cells[i].innerHTML = parseFloat(data[cm.dataIndex]).toFixed(cm.decimalPlaces);
				} else if (cm.dataType == 'checkbox' || cm.dataType == 'boolean') {
					var input_id = 'input_'+i+'x'+row;
					var control = $(input_id);
					if (control) {
						var idiv = control.getParent();
						var ldiv = idiv.getNext();
						var label = ldiv.getElement('label');
						if (data[cm.dataIndex] == cm.valueT || data[cm.dataIndex] == cm.labelT) {
							control.set('checked', true);
							if (label) {
								label.set('html', cm.labelT);
							}
						} else {
							control.set('checked', false);
							if (label) {
								label.set('html', cm.labelF);
							}
						}
					}
				} else if (cm.dataType == 'select') {
					var input_id = 'input_'+i+'x'+row;
					var control = $(input_id);
					if (control) {
						var iv = parseInt(data[cm.dataIndex]);
						if (isNaN(iv)) {
							control.set('value', data[cm.dataIndex]);
						} else if (iv >= 0 && iv < control.options.length) {
							control.selectedIndex = iv;
						}
					}
				} else {
					cells[i].innerHTML = data[cm.dataIndex];
				}
			}
		}
	},
	
	getScrollPos: function() {
		var bDiv = this.container.getElement('.bDiv');
		return {x:bDiv.scrollLeft, y:bDiv.scrollTop};
	},
	
	// API
	setScroll: function(x, y)
	{
		var bDiv = this.container.getElement('.bDiv');
		new Fx.Scroll(bDiv).set(x, y);
	},
	
	// API
	addRow: function(data, row){
		if (row >=0)
		{	
			// ako podataci nisu inic. napravi novi array
			if (!this.options.data)
				this.options.data = [];

			this.options.data.splice(row, 0, data);

			this.reset();
		}
	},
	
	// API
	deleteRow: function(row){
		if (row >=0 && row<this.options.data.length)
		{	
			this.options.data.splice(row, 1);
			this.reset();
		}
	},
	
	isHidden: function(i){
		return this.elements[i].hasClass( this.options.filterHideCls );
	},
	
	hideWhiteOverflow: function(i){
		if ( this.container.getElement('.gBlock') )
			this.container.getElement('.gBlock').dispose();
		
		var pReload = this.container.getElement('div.pDiv .pReload');
		if (pReload)
			pReload.removeClass('loading'); 
	},
	
	showWhiteOverflow: function(i){
		// ************* white overflow & loader ************
		if ( this.container.getElement('.gBlock') )
			this.container.getElement('.gBlock').dispose();
			
		var gBlock = new Element('div', {style:'top: 0px; left: 0px; background: white none repeat scroll 0% 0%; position: absolute; z-index: 999; opacity: 0.5'} ) ;
		var bDiv = this.container.getElement('.bDiv');
		
		var top = 1;
		top += this.container.getElement('.this.tDiv') ? this.container.getElement('.this.tDiv').getSize().y : 0;
		top += this.container.getElement('.hDiv') ? this.container.getElement('.hDiv').getSize().y : 0;
		
		// height: (bDiv && bDiv.getSize().y ? bDiv.getSize().y:this.options.height)
		gBlock.setStyles({width:this.options.width, height: this.options.height-1, top:0});
		gBlock.addClass('gBlock');
		
		this.container.appendChild(gBlock);
		
		var pReload = this.container.getElement('div.pDiv .pReload');
		if (pReload)
			pReload.addClass('loading');
	},
		
	
	showLoader: function(){
		if (this.loader)
			return;
		
		this.showWhiteOverflow(); // ovako bi cak mogao maknuti u sve staviti unutar showLoadera
		
		this.loader = new Element('div');
		
		this.loader.addClass('elementloader');
		this.loader.inject(this.container);
		
		// pozicioniranje loadera
		/*var gBlock = this.container.getElement('.gBlock');
		var sizeGBlock = gBlock.getSize();

		var top = 0;
		top += this.container.getElement('.this.tDiv') ? this.container.getElement('.this.tDiv').getSize().y : 0;
		top += this.container.getElement('.hDiv') ? this.container.getElement('.hDiv').getSize().y : 0;

		this.loader.setStyles({top:top+sizeGBlock.y/2-16, left: sizeGBlock.x/2});
		*/
		
		this.loader.setStyles({top:this.options.height/2-16, left:  this.options.width/2});
	},
	
	hideLoader: function(){
		if (!this.loader)
			return;
		
		this.hideWhiteOverflow();
		this.loader.dispose();
		this.loader = null;
		
	},
	
	// API
	selectAll: function(){

		this.elements.each(function(el, i){ 
			this.selected.push(el.retrieve('row'));
			el.addClass('selected'); 
		}, this);
	},
	
	// API
	unselectAll: function(){
		this.elements.each(function(el, i){ 
			el.removeClass('selected'); 
		}, this);
		
		this.selected = [];
	},
	
	// API
	getSelectedIndices: function(){
		return this.selected;
	},
	
	// API
	setSelectedIndices: function(arr){
		this.selected = arr;
		
		for (var i = 0; i < arr.length; i++) 
		{
			var li = this.elements[arr[i]];
			//el.addClass('selected');
			// simulate user click
			this.onRowClick({target:li.getFirst(), control:false}); 
		}

	},

	// mislim da je visak
	onMouseOver: function(obj){
		//alert(3);
		obj.columnModel.onMouseOver(obj.element, obj.data);
	},
	
	// API
	removeHeader: function(){
		var obj = this.container.getElement('.hDiv');
		
		if (obj) obj.empty();	//obj.set('html', '&nbsp;');
			
		this.options.columnModel = null;
	},	
	
	// API
	removeAll: function(){
		if (this.ulBody)
			this.ulBody.empty();
		
		this.selected = new Array();
		
		//this.options.data = null;
	},	
	
	// API
	setColumnModel: function(cmu){
		if ( !cmu )
			return;
				
		this.options.columnModel = cmu;
		this.adjustColumnModel();
		this.draw();
	},
	
	// API
	setColumnProperty: function(columnName, property, value){
	
		var cmu = this.options.columnModel;
		
		if ( !cmu || !columnName || !property  ) return;
		
		columnName = columnName.toLowerCase();
		
		for (var i=0; i<cmu.length; i++)
		{
			if ( cmu[i].dataIndex.toLowerCase() == columnName )
			{
				cmu[i][property] = value;
				
				return;
			}
		}
	},
	
	// Automatsko odredivanje column modela ako nije zadan
	setAutoColumnModel: function(){
	
		if ( !this.options.data ) return;
			
		var rowCount = this.options.data.length;
		
		if ( !(rowCount>0) )
			return;
			
		this.options.columnModel = [];
		
		// uzmi schemu od prvog podatka
		for ( var cn in this.options.data[0] )
		{
			var dataType = typeof(this.options.data[0][cn]) == "number" ? "number" : "string";
			this.options.columnModel.push({header:cn, dataIndex:cn, dataType: dataType, editable:true, decimalPlaces:2});
		}
		this.adjustColumnModel();
		
		this.fireEvent("autocolummodel", {target:this, columnModel:this.options.columnModel});
		
		this.draw();
	},
	
	// API
	setSize: function(w, h){
		
		// Width
		this.options.width = w ? w : this.options.width;
		
		this.container.setStyle('width', this.options.width);
		
		var width = this.options.width-2;
		if (this.options.buttons) this.container.getElement('.this.tDiv').setStyle('width', width);
		
		var hDiv = this.container.getElement('.hDiv');
		if (this.options.showHeader && hDiv) hDiv.setStyle('width', width);
		
		var bodyEl = this.container.getElement('.bDiv');
		bodyEl.setStyle('width', width);
		this.container.getElement('.pDiv').setStyle('width', width);
		
		// Height
		this.options.height = h ? h : this.options.height;

		bodyEl.setStyle('height', this.getBodyHeight() );
		this.container.setStyle('height', this.options.height);
		
		// ako je kojim slucajem whiteOverflow namjesti
		var gBlock = this.container.getElement('.gBlock');
		if (gBlock)
			gBlock.setStyles({width:this.options.width, height: bodyEl.getSize().y });
	},
	
	onBodyScroll: function(){
		var hbox = this.container.getElement('.hDivBox');
		
		var bbox = this.container.getElement('.bDiv');
		
		var xs = bbox.getScroll().x;
		
		//hbox.setStyle('position', 'relative');
		hbox.setStyle('left', -xs);

		this.rePosDrag();
		//console.debug(xs);
	},
	
	onBodyClick: function(){
		
	},	
	
	onBodyMouseOver: function(){
		//console.debug(this.onBodyScrollID);
		
	},	
	
	onBodyMouseOut: function(){
		
	},	
	
	// ************************************************************************
	// ************************* Drag columns events **************************
	// ************************************************************************
	
	rePosDrag: function(){
		if (!this.options.resizeColumns)
			return;
			
		var dragTempWidth = 0;
		var cDrags = this.container.getElements('.cDrag div');
		
		var scrollX = this.container.getElement('div.bDiv').getScroll().x;
		
		for (var c = 0; c < this.options.columnModel.length; c++) {
			var columnModel = this.options.columnModel[c];
			
			//if (columnModel.hidden) continue;
			
			// hidden-1
			var dragSt = cDrags[c];
		
			dragSt.setStyle('left', dragTempWidth+columnModel.width-scrollX);
			//console.log(dragTempWidth+columnModel.width+2);
			
			if (!columnModel.hidden)
				dragTempWidth += columnModel.width;
		}
	},
	
	onColumnDragComplete: function(target){
		this.dragging = false;
		
		var colindex = target.retrieve('column');
		
		// nadi poziciju prvo
		var cDrag = this.container.getElement('div.cDrag');
		var dragSt = cDrag.getElements('div')[colindex];
		var scrollX = this.container.getElement('div.bDiv').getScroll().x;
		
		// izracunaj nove ukupne duljine 
		this.sumWidth = 0;
		for (var c = 0; c < this.options.columnModel.length; c++) {
			var columnModel = this.options.columnModel[c];
			
			//if (columnModel.hidden) continue;

			if (c == colindex)
			{
				// nova vrijednost pomaknute kolone
				var pos = dragSt.getStyle('left').toInt()+scrollX-this.sumWidth; // zato sto je u dragSt.left +2
			}else if (!columnModel.hidden)			
				this.sumWidth += columnModel.width;
		}
		//console.log(pos);
		
		if (pos<30) // minimalna velicina kolone
			pos = 30
		
		this.options.columnModel[colindex].width = pos;
		
		this.sumWidth += pos;
		//console.log(this.sumWidth);
		
		this.ulBody.setStyle('width', this.sumWidth);
		var hDivBox = this.container.getElement('div.hDivBox');
		
		hDivBox.setStyle('width', this.sumWidth+this.visibleColumns*2);
	
		// header
		var columns = hDivBox.getElements('div.th');
		var columnObj = columns[colindex];
		
		columnObj.setStyle('width', pos-6);

		var visibleColumns = this.visibleColumns; // radi this. u each-u
		
		// radi accordiana
		var elements = this.ulBody.getElements('li.tr');

		// sve kolone u body
		elements.each(function(el, i){
			el.setStyle('width', this.sumWidth+2*visibleColumns); // inace se Div-ovi wrapaju
			
			if (!el.hasClass('section'))	
			{
				var columns = el.getElements('div.td');
				var columnObj = columns[colindex];
				columnObj.setStyle('width', pos-6);
			}
			
		});
		
		this.rePosDrag();		
	},
	
	onColumnDragStart: function(target){
		this.dragging = true;
	},
	
	onColumnDragging: function(target){
		target.setStyle('top', 1);
	},
	
	overDragColumn: function(evt){
		evt.target.addClass('dragging');
	},
	
	outDragColumn: function(evt){
		evt.target.removeClass('dragging');
	},
	
	// ************************************************************************
	// ************************* Header events ********************************
	// ************************************************************************

	clickHeaderColumn: function(evt){
		if (this.dragging) return;
		
		var colindex = evt.target.retrieve('column');
		var columnModel = this.options.columnModel[colindex];
		
		evt.target.removeClass(columnModel.sort);
		columnModel.sort = (columnModel.sort == 'ASC') ? 'DESC' : 'ASC';
		evt.target.addClass(columnModel.sort);

		//hidden-1
		this.sort(colindex);
	},
	
	overHeaderColumn: function(evt){
		if (this.dragging) return;
		
		var colindex = evt.target.retrieve('column');
		var columnModel = this.options.columnModel[colindex];

		evt.target.addClass(columnModel.sort);
	},
	
	outHeaderColumn: function(evt){
		if (this.dragging) return;
		
		var colindex = evt.target.retrieve('column');
		var columnModel = this.options.columnModel[colindex];
		
		evt.target.removeClass(columnModel.sort);
	},
	
	firstPage: function(){
		this.options.page = 1;		
		this.refresh();
	},
	
	prevPage: function(){
		if (this.options.page>1){
			this.options.page--;		
			this.refresh();
		}
	},
	
	nextPage: function(){

		if( (this.options.page+1) > this.options.maxpage)
			return;
		
		this.options.page++;		
		this.refresh();
	},
		
	lastPage: function(){
		this.options.page = this.options.maxpage;		
		this.refresh();
	},
	
	perPageChange: function(){
		this.options.page = 1;
		this.options.perPage = this.container.getElement('.rp').value;		
		this.refresh();
	},
		
	pageChange: function(){
		
		var np = this.container.getElement('div.pDiv2 input').value;
		
		if (np>0 && np<=this.options.maxpage)
		{
			if (this.refreshDelayID)
				$clear(this.refreshDelayID)
			
			this.options.page = np;
			
			this.refreshDelayID = this.refresh.delay(1000, this);
		}
	},
	
	// API
	gotoPage: function(p){
		if (p>0 && p<=this.options.maxpage)
		{
			this.options.page = p;
			
			this.refresh();
		}
	},
	
	setPerPage: function(p){
		if (p>0)
		{
			this.options.perPage = p;
			
			this.refresh();
		}
	},
	
	// API, not doc
	sort: function(index, by){
		
		if ( index<0 || index>=this.options.columnModel.length )
			return;

		if(this.options.onStart){
			this.fireEvent('onStart');
		}
		
		//
		var header = this.container.getElements('.th');
		var el = header[index];
		
		if (by != null)
			el.addClass(by.toLowerCase());
		
		if(el.hasClass('ASC')){
			el.sortBy = 'ASC';
		}else if(el.hasClass('DESC')){
			el.sortBy = 'DESC';
		}
		
		if (this.options.serverSort){
			this.options.sortOn = this.options.columnModel[index].dataIndex;
			this.options.sortBy = el.sortBy;
			
			this.refresh();
		}else{
			// Sorting...
			this.elements.sort(el.compare);
			this.elements.injectInside(this.ulBody);
			
			// Update selection array because indices has been changed
			this.selected = new Array();
			this.elements.each(function(el ,i){
				if(el.hasClass('selected')){
					this.selected.push(el.retrieve('row'));
				}
			}, this);
			
			// Filter
			if (this.filtered) {
				this.filteredAltRow();
			} else {
				this.altRow();
			}
		}	
	},
	
	altRow: function(){
		this.elements.each(function(el,i){
			if(i % 2){
				el.removeClass('erow');
			}else{
				el.addClass('erow');
			}
		});
	},
	
	filteredAltRow: function(){
		this.ulBody.getElements('.'+this.options.filterSelectedCls).each(function(el,i){
			if(i % 2){
				el.removeClass('erow');
			}else{
				el.addClass('erow');
			}
		});
	},

	getRowCount: function() {
		return this.options.data.length;
	},
	
	getColumnCount: function() {
		return this.columnCOunt;
	},

	getVisibleColumnCount: function() {
		return this.visibleColumns;
	},
	
	getCell: function(colnumber, rownumber) {
		var grid = this.container;
		var rows = grid.getElements('div.bDiv ul li');
		if (rows && rows.length > 0) {
			var cells = rows[rownumber].getElements('div');
			return cells[colnumber];
		}
		return null;
	}
	
});

var OmniGrid = new Class({
	Extends: BaseGrid,
	
	getOptions: function() {
		var parentOptions = this.parent();
		var options = {
			selectable:true,
			serverSort:true,
			sortOn: null,
			sortBy: 'ASC',
			filterHide: true,
			filterHideCls: 'hide',
			filterSelectedCls: 'filter',
			multipleSelection:true,
			editable:false,
			// accordion
			accordion:false,
			accordionRenderer:null,
			autoSectionToggle:true, // if true just one section can be open/visible
			showToggleIcon:true,
			openAccordionOnDblClick:false,
			filterInput:false,
		};
		return $merge({}, parentOptions, options);
	},

	initialize: function(container, options, data){
		options = $merge({}, this.getOptions(), options);
		this.parent(container, options, data);
	},
	
	createHeader: function() {
		var headerDiv = this.parent();
		// ********************** Header events **************************
		if (this.options.sortHeader)
		{
			var hDivBox = headerDiv.getElements('div.hDivBox')[0];
			var headerCells = hDivBox.getElements('div');
			for (var i = 0; i < headerCells.length; i++) {
				var div = headerCells[i];
				div.addEvent('click', this.clickHeaderColumn.bind(this));
				div.addEvent('mouseout', this.outHeaderColumn.bind(this));
				div.addEvent('mouseover', this.overHeaderColumn.bind(this));
			}
		}
		return headerDiv;
	},
	
	renderData: function(){
		this.parent();		
		if (this.options.accordion)	
			this.elements = this.ulBody.getElements('li:nth-child(2n+1)') // all li el. except accordian sections
		else
			this.elements = this.ulBody.getElements('li');
	},

	renderDataRowCreated: function(li, rowNumber) {
		if (this.options.accordion)
		{
			/*	var div = new Element('div');
				div.addClass('section');
				li.appendChild(div);
			*/
			var li2 = new Element('li');
			li2.addClass('section');
			li2.addClass('section-'+r);
			li2.setStyle('width', this.sumWidth+2*this.visibleColumns); // inace se Div-ovi wrapaju, a u IE nastaje cudan 1px border ispod LI el.
			
			this.ulBody.appendChild(li2);
			
			if (this.options.accordionRenderer)	
				this.options.accordionRenderer({parent:li2, row:r, grid:this, rowdata: rowdata});
		}
	},
	
	reset: function() {		
		this.filtered = false;
		this.lastsection = null;
		this.parent();
	},
	
	// API
	getSection: function(row){
		return this.ulBody.getElement('.section-'+row);
	},
	
	onRowClick: function (evt){	
		this.parent(evt);
		if (this.options.accordion && !this.options.openAccordionOnDblClick)
		{
			var li = this.getLiParent(evt.target);
			this.accordianOpen(li);
		}
		
		this.fireEvent("click", {indices:this.selected, target:this, row:li.retrieve('row'), element:li });
	},

	onKeyDown: function (evt){
		var bDiv = this.container.getElement('.bDiv');
		
		switch (evt.key) {
			case 'enter':
				if (!this.inlineeditmode) {
					this.edit();
				}
				break;
			/*
			case 'up':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.activeCell.column, this.activeCell.row-1);
				break;
			case 'down':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.activeCell.column, this.activeCell.row+1);
				break;
				**/
		}
		this.fireEvent("keydown", {target:this });
	},

	toggleIconClick: function(evt)
	{
		var li = this.getLiParent(evt.target);
		
		this.accordianOpen(li);
	},
	
	accordianOpen: function(li)
	{
		var section = this.getSection(li.retrieve('row'));
	
		if (this.options.autoSectionToggle)
		{
			
			if (this.lastsection)
				if (this.lastsection != section)
				{
					this.lastsection.setStyle('display', 'none');
					this.lastsection.getPrevious().getElement('.toggleicon').setStyle('background-position', '0 0');
				}
			
			if (!this.options.accordionRenderer)
			{
				section.setStyle('display', 'block');
			}
		}
		
		if (this.options.accordionRenderer)
		{
			this.toggle( section );
		}
		
		
		if (this.options.showtoggleicon)
		{
			li.getElement('.toggleicon').setStyle('background-position', section.getStyle('display') == 'block' ? '-16px 0' : '0 0');
		}
		
		
		this.lastsection = section;
	},
	
	// API
	filter: function(form){
		//var form = $(form);
		var col = 0;
		var key = '';
		
		if ( !(form.length>0) )
			this.clearFilter();
		
		
		key = form;
		
		if(key)
		{			
			for (var i=0; i<this.options.data.length; i++)
			{
				var dat = this.options.data[i];
			
				for (var c=0; c<this.options.columnModel.length; c++)
				{
					var columnModel = this.options.columnModel[c];
					
					if ( columnModel.type == "checkbox")
						continue;
					
					var el = this.elements[i];
					
					if(this.options.filterHide){
						el.removeClass('erow');
					}
					
					if(dat[columnModel.dataIndex] != null && dat[columnModel.dataIndex].toLowerCase().indexOf(key) > -1)
					{
						el.addClass(this.options.filterSelectedCls);
						if(this.options.filterHide){
							el.removeClass(this.options.filterHideCls);
						}
						
						break;
					}else{
						el.removeClass(this.options.filterSelectedCls);
						if(this.options.filterHide){
							el.addClass(this.options.filterHideCls);
						}
					}
				}				
			}
			
			if(this.options.filterHide){
				this.filteredAltRow();
				this.filtered = true;
			}
		}
	},
	
	// API
	clearFilter: function(){
		this.elements.each(function(el,i){
			el.removeClass(this.options.filterSelectedCls);
			if(this.options.filterHide){
				el.removeClass(this.options.filterHideCls);
			}
		}, this);
		if(this.options.filterHide){
			this.altRow();
			this.filtered = false;
		}
	},
});


var ActiveGrid = new Class({
	Extends: BaseGrid,
	
	getOptions: function() {
		var parentOptions = this.parent();
		parentOptions.selectable = false;
		parentOptions.multipleSelection = false;
		var options = {};
		return $merge({}, parentOptions, options);
	},
	
	initialize: function(container, options, data) {
		options = $merge({}, this.getOptions(), options);
		this.parent(container, options, data);
		this.scrollData = new Object();
		if (this.options.data) {
			this.activeCell = {column:0, row:0, div:null};
			if (this.options.columnModel[0].hidden) {
				this.activeCell.column = this.getNextVisibleColumn(0);
			}
			this.activeCell.div = this.getCell(this.activeCell.column, this.activeCell.row);
			this.setActiveCell(this.activeCell.column, this.activeCell.row);
		}
	},
	
	getEditorParams: function(options) {
		var editorParams = {c:null, row:null, li:null, colmod:null, td:null, data:null};
		editorParams.c = this.activeCell.column;
		editorParams.row = this.activeCell.row;
		editorParams.li = this.elements[editorParams.row];
		editorParams.colmod = this.options.columnModel[editorParams.c];
		if (!editorParams.colmod.editable) return editorParams;
		editorParams.td = editorParams.li.getElements('div.td')[editorParams.c];
		editorParams.data = this.options.data[editorParams.row];
		return editorParams;
	},
	
	onCellClick: function(div, evt) {
		this.parent(div,evt);
		this.setActiveCell(div.retrieve('column'), div.retrieve('row'));
	},

	onRowClick: function (evt){	
		this.parent(evt);
		var li = this.getLiParent(evt.target);
		this.fireEvent("click", {indices:this.selected, target:this, row:li.retrieve('row'), element:li });
	},

	onKeyDown: function (evt){
		var bDiv = this.container.getElement('.bDiv');
		
		switch (evt.key) {
			case 'enter':
				if (!this.inlineeditmode) {
					this.edit();
				}
				break;
			case 'left':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.getPrevVisibleColumn(this.activeCell.column), this.activeCell.row);
				break;
			case 'right':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.getNextVisibleColumn(this.activeCell.column), this.activeCell.row);
				break;
			case 'up':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.activeCell.column, this.activeCell.row-1);
				break;
			case 'down':
				if (this.inlineeditmode || !this.activeCell) break;
				evt.stop();
				this.setActiveCell(this.activeCell.column, this.activeCell.row+1);
				break;
		}
		this.fireEvent("keydown", {target:this });
	},

	setActiveCell: function(column, row)
	{
		if (!this.options.data) {
			return;
		}
		
		column = parseInt(column);
		row = parseInt(row);
		
		if (isNaN(column) || isNaN(row)) {
			return;
		}
		
		var columnCount = this.columnCount;
		var rowCount = this.options.data.length;

		if (column >= 0 && column < columnCount && row >= 0 && row < rowCount) {
			var scrPos = this.getScrollPos();
			var scr_x = parseInt(scrPos.x);
			var scr_y = parseInt(scrPos.y);
			
			var c_dt = Math.abs(this.activeCell.column-column);
			var r_dt = Math.abs(this.activeCell.row-row);
			this.activeCell.column = column;
			this.activeCell.row = row;
			
			var cell_w = this.options.columnModel[column].width;
			var start_cell_x = column > 0 ? this.sumWidths[column-1] : 0;
			var end_cell_x = this.sumWidths[column];
			
			var start_cell_y = row > 0 ? this.sumHeights[row-1] : 0;
			var end_cell_y = this.sumHeights[row];
			
			var horizontalScrollbarHeight = 0;
			var verticalScrollbarWidth = 0;			
			if (this.sumWidth > this.options.width) {
				horizontalScrollbarHeight = 16;
			}
			if (this.sumHeight > this.options.height) {
				verticalScrollbarWidth = 15;
			}
			
			var headerRowHeight = 0;
			if (this.options.showHeader) {
				headerRowHeight = 27;
			}

			// Compute scr_x offset
			var visibleAreaMaxX = scrPos.x + this.options.width ;
			if (scrPos.x > start_cell_x || end_cell_x > visibleAreaMaxX) {
				if (visibleAreaMaxX < end_cell_x) {
					var dt = end_cell_x - visibleAreaMaxX + verticalScrollbarWidth;
					scr_x = scrPos.x + dt;
				}
				if (scrPos.x > start_cell_x) {
					scr_x = start_cell_x;
				}
			}
			
			// Compute scr_y offset
			var visibleAreaMaxY = scrPos.y + (this.options.height-headerRowHeight) - horizontalScrollbarHeight;
			if (scrPos.y > start_cell_y || end_cell_y > visibleAreaMaxY) {
				if (visibleAreaMaxY < end_cell_y) {
					var dt = end_cell_y - visibleAreaMaxY + horizontalScrollbarHeight;
					scr_y = scrPos.y + dt;
				}
				if (scrPos.y > start_cell_y) {
					scr_y = start_cell_y;
				}
			}
			
			this.setScroll(scr_x, scr_y);
			/*
			scrPos = this.getScrollPos();
			$('log').set('text', 'column: ' + column + ', row: ' + row + ', c_dt: ' + c_dt + ', r_dt: ' + r_dt +
						', end_cell_x: ' + end_cell_x + ', end_cell_y: ' + end_cell_y + 
						',  scr_x: ' + scr_x + ', scr_y: ' + scr_y +
						', scrollLeft: ' + scrPos.x + ', scrollTop: ' + scrPos.y);
						*/
			var rows = this.ulBody.getElements('li');
			var cells = rows[row].getElements('div.td');
			if (this.activeCell.div) {
				this.activeCell.div.removeClass('activetd');
				this.activeCell.div = cells[column];
				this.activeCell.div.addClass('activetd');
			}
			
		}
	},

	renderData: function(){
		this.parent();		
		this.elements = this.ulBody.getElements('li');
	},

	setData: function(data, cm) {
		this.parent(data, cm);
		if (this.bodyDiv) {
			this.activeCell = {column:0, row:0, div:null};
			if (this.options.columnModel[0].hidden) {
				this.activeCell.column = this.getNextVisibleColumn(0);
			}
			this.activeCell.div = this.getCell(this.activeCell.column, this.activeCell.row);
			this.setActiveCell(this.activeCell.column, this.activeCell.row);
		}
	},
	
	finishEditing: function(evt) {
		this.parent(evt);
		if (evt && evt.type == "keydown" && evt.key == 'enter') {
			// Move active cell to next cell below current one
			var bDiv = this.container.getElement('.bDiv');
			var rowCount = this.options.data.length;
			this.setActiveCell(this.activeCell.column, this.activeCell.row+1);
		}
	}
});
